package io.yarsa.serial_nizi_pos

import androidx.compose.material3.OutlinedTextField
import android.annotation.SuppressLint
import android.content.Context
import android.hardware.usb.UsbManager
import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.material3.Button
import androidx.compose.material3.CircularProgressIndicator
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.ui.Modifier
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import com.hoho.android.usbserial.driver.UsbSerialPort
import com.hoho.android.usbserial.driver.UsbSerialProber
import io.yarsa.serial_nizi_pos.ui.theme.NiziPOSTheme

class MainActivity : ComponentActivity() {
    private var serialPort: UsbSerialPort? = null
    private var isLoading = mutableStateOf(true)

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        initSerial()

        setContent {
            NiziPOSTheme {
                Scaffold(modifier = Modifier.fillMaxSize()) { innerPadding ->
                    SerialScreen(
                        onSend = { sendCommand(it) },
                        isLoading = isLoading.value
                    )
                }
            }
        }
    }

    @SuppressLint("ServiceCast")
    private fun initSerial() {
        val manager = getSystemService(Context.USB_SERVICE) as UsbManager
        val drivers = UsbSerialProber.getDefaultProber().findAllDrivers(manager)
        if (drivers.isEmpty()) return

        val driver = drivers[0]
        val connection = manager.openDevice(driver.device) ?: return
        val port = driver.ports[0]

        port.open(connection)
        port.setParameters(115200, 8, UsbSerialPort.STOPBITS_1, UsbSerialPort.PARITY_NONE)

        serialPort = port
        isLoading.value = false
    }

    private fun sendCommand(command: String) {
        val port = serialPort ?: return
        val commando = command + "\n";
        Thread {
            port.write(commando.toByteArray(), 1000)
        }.start()
    }
}

val commands = listOf(
    Command("logoCommand", "Display Logo", "IDLE", "FSREAD**test.txt"),
    Command("textDisplayCommand", "Text Display", "TEXT**Main Title**Subtitle Text**Message", "TEXT**Topic**Title**Message"),
    Command("qrCommand", "QR Payload", "QR**Rs. 123.45**QRPayloadData", "QR**Amount**PayloadData"),
    Command("loadingCommand", "Loading Command", "WAIT**Rs. 560.50**Please wait...", "WAIT**Amount**Message"),
    Command("successCommand", "Success Command", "PASS**SUCCESS!**Payment successful", "PASS**Title**Message"),
    Command("failCommand", "Failure Command", "FAIL**Rs. 560.50**Payment Failed", "FAIL**Amount**Message"),
    Command("warnCommand", "Warning", "WARN**Device Not Ready**Please wait", "WARN**Title**Message"),
    Command("infoCommand", "Information", "INFO**Important**Keep device connected", "INFO**Title**Message"),
    Command("sleepCommand", "Sleep Device", "RESET", "RESET"),
    Command("formatCommand", "Format Image", "FORMAT", "FORMAT"),
    Command("wakeCommand", "Wake Device", "WAKE", "WAKE"),
    Command("screentimeCommand", "Screen Time", "SCREENTIME**60", "SCREENTIME**<time in seconds (30-300)>"),
)

data class Command(val id: String, val label: String, val defaultValue: String, val placeholder: String)

@Composable
fun SerialScreen(
    onSend: (String) -> Unit,
    isLoading: Boolean
) {
    Column(modifier = Modifier.padding(16.dp)) {
        if (isLoading) {
            CircularProgressIndicator()
        } else {
            val commandStates = remember { commands.associate { it.id to mutableStateOf(it.defaultValue) } }
            val scrollState = rememberScrollState()

            Column(modifier = Modifier.verticalScroll(scrollState)) {
                commands.forEach { cmd ->
                    val valueState = commandStates[cmd.id]!!
                    OutlinedTextField(
                        value = valueState.value,
                        onValueChange = { valueState.value = it },
                        label = { Text(cmd.label) },
                        placeholder = { Text(cmd.placeholder) },
                        modifier = Modifier.fillMaxWidth()
                    )
                    Button(
                        onClick = { onSend(valueState.value) },
                        modifier = Modifier.fillMaxWidth().padding(vertical = 4.dp)
                    ) {
                        Text("Send")
                    }
                }
            }

        }
    }
}